<?php
//BambooBasic Graphics Library  [graphics.php]
//Copyright (c) 2012 Michael Denathorn
//This software is protected by copyright, and must not be distributed by any means other then what is outlined in your End User License Agreement (EULA)
//No warranty implied, use at your own risk.

//Creation
function bam_createimage($iWidth, $iHeight)
{
	return(imagecreatetruecolor($iWidth, $iHeight));
}

function bam_loadimage($sFilename)
{
	$ext = strtolower(pathinfo($sFilename, PATHINFO_EXTENSION));
	
	switch($ext)
	{
		case "png":
			return(imagecreatefrompng($sFilename));
			break;
		case "jpg":
			return(imagecreatefromjpeg($sFilename));
			break;
		case "gif":
			return(imagecreatefromgif($sFilename));
			break;
		default:
			return(false);
	}
}

function bam_copyimage(&$resDestImage, $iDestX, $iDestY, $resSourceImage, $iSourceX, $iSourceY, $iSourceWidth, $iSourceHeight)
{
	return(imagecopy($resDestImage,$resSourceImage,$iDestX,$iDestY,$iSourceX,$iSourceY,$iSourceWidth,$iSourceHeight));
}

function bam_rotateimage($resImage, $fAngle, $iBackgroundColor, $iIgnoreTransparent = 0)
{
	return(imagerotate($resImage, $fAngle, $iBackgroundColor, $iIgnoreTransparent));
}

function bam_destroyimage($resImage)
{
	return(imagedestroy($resImage));
}

//Settings
function bam_allocatergba(&$resImage, $iRed, $iGreen, $iBlue, $iAlpha = 0)
{
	return(imagecolorallocatealpha($resImage, $iRed, $iGreen, $iBlue, $iAlpha));
}

function bam_deallocatergba(&$resImage, $iColor)
{
	return(imagecolordeallocate($resImage, $iColor));
}

function bam_setalphablend(&$resImage, $bAlphaBlend)
{
	return(imagealphablending($resImage, $bAlphaBlend));
}

function bam_setlineweight(&$resImage, $iWeight)
{
	return(imagesetthickness($resImage, $iWeight));
}

function bam_setimagestyle(&$resImage, $arrStyle)
{
	return(imagesetstyle($resImage,$arrStyle));
}

function bam_savealpha(&$resImage, $bFlag)
{
	return(imagesavealpha($resImage, $bFlag));
}

//Shapes
function bam_drawrect(&$resImage,$iX, $iY, $iWidth, $iHeight, $iColor, $iFilled = False)
{
	if($iFilled == True)
	{
		return(imagefilledrectangle($resImage, $iX, $iY, ($iX + $iWidth), ($iY+$iHeight), $iColor));
	}
	else
	{
		return(imagerectangle($resImage,$iX, $iY, ($iX + $iWidth), ($iY+$iHeight), $iColor));
	} 
}

function bam_drawline(&$resImage, $iX1, $iY1, $iX2, $iY2, $iColor)
{
	imageline($resImage, $iX1, $iY1, $iX2, $iY2, $iColor);
}

function bam_drawoval(&$resImage,$iX, $iY, $iWidth, $iHeight, $iColor, $iFilled = False)
{
	if($iFilled == True)
	{
		return(imagefilledellipse($resImage, ($iX + ($iWidth/2)), ($iY + ($iHeight/2)), $iWidth, $iHeight, $iColor));
	}
	else
	{
		return(imageellipse($resImage, ($iX + ($iWidth/2)), ($iY + ($iHeight/2)), $iWidth, $iHeight, $iColor));
	} 
}

function bam_drawarc(&$resImage,$iCX, $iCY, $iWidth, $iHeight, $iStart, $iEnd, $iColor, $iStyle = IMG_ARC_PIE, $iFilled = False)
{
	if($iFilled == True)
	{
		return(imagefilledarc($resImage,$iCX, $iCY, $iWidth, $iHeight, $iStart, $iEnd, $iColor, $iStyle));
	}
	else
	{
		return(imagearc($resImage,$iCX, $iCY, $iWidth, $iHeight, $iStart, $iEnd, $iColor));
	} 
}

function bam_drawpolygon(&$resImage, $arrPoints, $iNumPoints, $iColor, $iFilled = False)
{
	if($iNumPoints < 3)
	{
		return(False);
	}
	
	if($iFilled == True)
	{
		return(imagefilledpolygon($resImage, $arrPoints, $iNumPoints, $iColor));
	}
	else
	{
		return(imagepolygon($resImage, $arrPoints, $iNumPoints, $iColor));
	}
}

function bam_drawtext(&$resImage, $sText, $iX, $iY, $iColor, $iFont = 2, $iVertical = False)
{
	if($iVertical == true)
	{
		$fontWidth = imagefontwidth($iFont);
		$textWidth = strlen($sText) * $fontWidth;
		return(imagestringup($resImage, $iFont, $iX, $iY + $textWidth, $sText, $iColor));
	}
	else
	{
		return(imagestring($resImage, $iFont, $iX, $iY, $sText, $iColor));
	}
}

function bam_drawttftext(&$resImage, $sText, $iX, $iY, $iColor, $fSize, $sFontFile, $fAngle = 0.0)
{
	return(imagettftext($resImage,$fSize,$fAngle,$iX,$iY,$iColor,$sFontFile,$sText));
}

function bam_fillimage(&$resImage, $iX, $iY, $iColor)
{
	return(imagefill($resImage,$iX,$iY,$iColor));
}

function bam_setpixel(&$resImage, $iX, $iY, $iColor)
{
	return(imagesetpixel($resImage, $iX, $iY, $iColor));
}

//Fonts
function bam_loadimagefont($sFilename)
{
	return(imageloadfont($sFilename));
}

function bam_fontwidth($iFont)
{
	return(imagefontwidth($iFont));
}

function bam_fontheight($iFont)
{
	return(imagefontheight($iFont));
}

function bam_ttftextsizeinfo($sText, $fSize, $sFontFile, $fAngle = 0)
{
  $box   = imagettfbbox($fSize, $fAngle, $sFontFile, $sText); 
  if( !$box ) 
    return false; 
  $min_x = min( array($box[0], $box[2], $box[4], $box[6]) ); 
  $max_x = max( array($box[0], $box[2], $box[4], $box[6]) ); 
  $min_y = min( array($box[1], $box[3], $box[5], $box[7]) ); 
  $max_y = max( array($box[1], $box[3], $box[5], $box[7]) ); 
  $width  = ( $max_x - $min_x ); 
  $height = ( $max_y - $min_y ); 
  $left   = abs( $min_x ) + $width; 
  $top    = abs( $min_y ) + $height; 
  $img     = @imagecreatetruecolor( $width << 2, $height << 2 ); 
  $white   =  imagecolorallocate( $img, 255, 255, 255 ); 
  $black   =  imagecolorallocate( $img, 0, 0, 0 ); 
  imagefilledrectangle($img, 0, 0, imagesx($img), imagesy($img), $black); 
  imagettftext( $img, $fSize, 
                $fAngle, $left, $top, 
                $white, $sFontFile, $sText); 
  $rleft  = $w4 = $width<<2; 
  $rright = 0; 
  $rbottom   = 0; 
  $rtop = $h4 = $height<<2; 
  for( $x = 0; $x < $w4; $x++ ) 
    for( $y = 0; $y < $h4; $y++ ) 
      if( imagecolorat( $img, $x, $y ) ){ 
        $rleft   = min( $rleft, $x ); 
        $rright  = max( $rright, $x ); 
        $rtop    = min( $rtop, $y ); 
        $rbottom = max( $rbottom, $y ); 
      } 
  imagedestroy( $img ); 
  return array( 0   => $left - $rleft, 
                1    => $top  - $rtop, 
                2  => $rright - $rleft + 1, 
                3 => $rbottom - $rtop + 1 ); 
}

//Properties
function bam_imagewidth($resImage)
{
	return(imagesx($resImage));
}

function bam_imageheight($resImage)
{
	return(imagesy($resImage));
}

function bam_getpixel($resImage, $iX, $iY)
{
	return(imagecolorat($resImage, $iX, $iY));
}


//Export
function bam_saveimage($resImage, $sFilename, $iQuality = 0)
{
	$ext = strtolower(pathinfo($sFilename, PATHINFO_EXTENSION));
	
	switch($ext)
	{
		case "png":
			return(imagepng($resImage, $sFilename, $iQuality));
			break;
		case "jpeg":
			return(imagejpeg($resImage, $sFilename, $iQuality));
			break;
		case "gif":
			return(imagegif($resImage, $sFilename));
			break;
		default:
			return(false);
		
	}
	return(imagepng($resImage,$sFilename, $iQuality));
}
?>